package ru.bitel.bgbilling.modules.inet.dyn.device.dns;

import java.util.Collections;
import java.util.Set;

import ru.bitel.bgbilling.kernel.network.dns.DnsClient;
import ru.bitel.bgbilling.modules.inet.access.sa.ServiceActivator;
import ru.bitel.bgbilling.modules.inet.access.sa.ServiceActivatorAdapter;
import ru.bitel.bgbilling.modules.inet.access.sa.ServiceActivatorEvent;
import ru.bitel.bgbilling.modules.inet.api.common.bean.InetConnection;
import ru.bitel.bgbilling.modules.inet.api.common.bean.InetDevice;
import ru.bitel.bgbilling.modules.inet.api.common.bean.InetDeviceType;
import ru.bitel.bgbilling.server.util.Setup;
import ru.bitel.common.ParameterMap;
import ru.bitel.common.Utils;
import ru.bitel.common.inet.IpAddress;

/**
 * Динамический DNS для соединений. Либо всегда привязывает DNS, если есть устройство в дереве с данным ServiceActivator,
 * либо если установлен список опций Inet - dns.inetOptions= - то при наличии активной опции (т.е. в этом случае, например, в тарифе указывается опция Inet
 * при активности определенной тарифной опции - тарифную опцию активировали - DNS привязался, деактивировали - отвязался). 
 * @author amir
 *
 */
public class DynDnsServiceActivator
	extends ServiceActivatorAdapter
	implements ServiceActivator
{
	private DnsClient dnsClient;

	/**
	 * Опция или опции при которых привязываем DNS. Если null - значит всегда привязываем (если данный ServiceActivator установлен).
	 */
	private Set<Integer> options;

	@Override
	public Object init( Setup setup, int moduleId, InetDevice device, InetDeviceType deviceType, ParameterMap config )
		throws Exception
	{
		super.init( setup, moduleId, device, deviceType, config );

		dnsClient = new DnsClient( config );

		options = Utils.toIntegerSet( config.get( "dns.inetOptions", null ) );
		if( options.size() == 0 )
		{
			options = null;
		}

		return null;
	}

	@Override
	public Object destroy()
		throws Exception
	{
		dnsClient.close();
		dnsClient = null;

		return super.destroy();
	}

	private String getHostname( final InetConnection connection )
	{
		String login = connection.getUsername();

		if( Utils.isBlankString( login ) )
		{
			return null;
		}

		return login.replace( '@', '-' ).toLowerCase();
	}

	private void register( final ServiceActivatorEvent e )
	{
		final InetConnection connection = e.getConnection();
		final String hostname = getHostname( connection );

		if( hostname != null )
		{
			dnsClient.registerAsync( hostname, IpAddress.toString( connection.getInetAddressBytes() ) );
		}
	}

	private void unregister( final ServiceActivatorEvent e )
	{
		final InetConnection connection = e.getConnection();
		final String hostname = getHostname( connection );

		if( hostname != null )
		{
			dnsClient.unregisterAsync( hostname );
		}
	}

	@Override
	public Object onAccountingStart( final ServiceActivatorEvent e )
		throws Exception
	{
		if( options != null && Collections.disjoint( e.getNewOptions(), options ) )
		{
			return null;
		}

		register( e );

		return null;
	}

	@Override
	public Object onAccountingStop( final ServiceActivatorEvent e )
		throws Exception
	{
		if( options != null && Collections.disjoint( e.getNewOptions(), options ) )
		{
			return null;
		}

		unregister( e );

		return null;
	}

	@Override
	public Object connectionModify( final ServiceActivatorEvent e )
		throws Exception
	{
		if( options == null )
		{
			return null;
		}

		if( Collections.disjoint( e.getOldOptions(), options ) )
		{
			if( !Collections.disjoint( e.getNewOptions(), options ) )
			{
				register( e );
			}
		}
		else
		{
			if( Collections.disjoint( e.getNewOptions(), options ) )
			{
				unregister( e );
			}
		}

		return null;
	}
}
